@@ -12,12 +12,14 @@ import time
 from base64 import b64encode
 from collections import deque
 from functools import partial
+from operator import itemgetter
 from new_reader import reader
 from threefive import Stream, Cue
+from iframes import IFramer
 
 MAJOR = "0"
 MINOR = "1"
-MAINTAINENCE = "29"
+MAINTAINENCE = "37"
 
 
 def version():
@@ -35,15 +37,6 @@ def version():
     return f"{MAJOR}.{MINOR}.{MAINTAINENCE}"
 
 
-def version_number():
-    """
-    version_number returns version as an int.
-    if version() returns 2.3.01
-    version_number will return 2301
-    """
-    return int(f"{MAJOR}{MINOR}{MAINTAINENCE}")
-
-
 class SegData:
     """
     A SegData instance is used to keep hold
@@ -80,6 +73,7 @@ class SCTE35:
         """
         if cue:
             return f'#EXT-X-SCTE35:CUE="{b64encode(cue.bites).decode()}"'
+        return "# No Cue"
 
     @staticmethod
     def is_cue_out(cue):
@@ -125,8 +119,18 @@ class X9K3(Stream):
 
     # sliding window size
     WINDOW_SLOTS = 5
-    # target segment time.
-    SECONDS = 2
+    __slots__ = [
+        "_tsdata",
+        "start",
+        "window_slot",
+        "header",
+        "live",
+        "output_dir",
+        "delete",
+        "sidecar",
+        "seconds",
+        "replay",
+    ]
 
     def __init__(self, tsdata=None, show_null=False):
         """
@@ -146,9 +150,12 @@ class X9K3(Stream):
         self.output_dir = "."
         self.delete = False
         self.seg = SegData()
+        self.sidecar_file = None
         self.sidecar = None
+        self.seconds = 2
         self.replay = False
         self._parse_args()
+        self.iframer = IFramer()
 
     def _parse_args(self):
         """
@@ -177,7 +184,15 @@ class X9K3(Stream):
             "-s",
             "--sidecar",
             default=None,
-            help="""sidecar file of scte35 cues. each line contains  PTS, Cue""",
+            help="""Sidecar file of scte35 cues. each line contains  PTS, Cue""",
+        )
+
+        parser.add_argument(
+            "-t",
+            "--time",
+            default=2,
+            type=float,
+            help="Segment time in seconds ( default is 2)",
         )
 
         parser.add_argument(
@@ -219,7 +234,8 @@ class X9K3(Stream):
         args = parser.parse_args()
         self._apply_args(args)
 
-    def _args_version(self, args):
+    @staticmethod
+    def _args_version(args):
         if args.version:
             print(version())
             sys.exit()
@@ -245,7 +261,11 @@ class X9K3(Stream):
 
     def _args_sidecar(self, args):
         if args.sidecar:
-            self.load_sidecar(args.sidecar)
+            self.sidecar = deque()
+            self.sidecar_file = args.sidecar
+
+    def _args_time(self, args):
+        self.seconds = args.time
 
     def _apply_args(self, args):
         """
@@ -255,8 +275,9 @@ class X9K3(Stream):
         self._args_version(args)
         self._args_input(args)
         self._args_output_dir(args)
-        self._args_flags(args)
         self._args_sidecar(args)
+        self._args_time(args)
+        self._args_flags(args)
         if isinstance(self._tsdata, str):
             self._tsdata = reader(self._tsdata)
 
@@ -284,7 +305,7 @@ class X9K3(Stream):
         if not self.live:
             play_type = "#EXT-X-PLAYLIST-TYPE:VOD"
             headers.append(play_type)
-        target = f"#EXT-X-TARGETDURATION:{self.SECONDS+self.SECONDS}"
+        target = f"#EXT-X-TARGETDURATION:{int(self.seconds+1)}"
         headers.append(target)
         seq = f"#EXT-X-MEDIA-SEQUENCE:{self.seg.start_seg_num}"
         headers.append(seq)
@@ -292,18 +313,23 @@ class X9K3(Stream):
         headers.append(bumper)
         self.header = "\n".join(headers)
 
-    def load_sidecar(self, file):
+    def load_sidecar(self, file, pid):
         """
         load_sidecar reads (pts, cue) pairs from
         the sidecar file and loads them into X9K3.sidecar
         """
-        self.sidecar = deque()
-        with reader(file) as sidefile:
-            for line in sidefile:
-                line = line.decode().strip().split("#", 1)[0]
-                if len(line):
-                    pts, cue = line.split(",", 1)
-                    self.sidecar.append([float(pts), cue])
+        if self.sidecar_file:
+            with reader(file) as sidefile:
+                for line in sidefile:
+                    line = line.decode().strip().split("#", 1)[0]
+                    if len(line):
+                        pts, cue = line.split(",", 1)
+                        if float(pts) >= self.pid2pts(pid):
+                            if [float(pts), cue] not in self.sidecar:
+                                self.sidecar.append([float(pts), cue])
+                                self.sidecar = deque(
+                                    sorted(self.sidecar, key=itemgetter(0))
+                                )
 
     def chk_sidecar_cues(self, pid):
         """
@@ -341,20 +367,18 @@ class X9K3(Stream):
         """
         self.scte35.cue.show()
         print(f"{self.scte35.cue.command.name}")
-        self.active_data.write(
-            f"# {self.scte35.cue.command.name} @ {self.scte35.cue.command.pts_time}\n"
-        )
         if "pts_time" in self.scte35.cue.command.get():
             self.scte35.cue_time = self.scte35.cue.command.pts_time
             print(
                 f"Preroll: {round(self.scte35.cue.command.pts_time- self.pid2pts(pid), 6)} "
             )
+            self.scte35.cue_tag = self.scte35.mk_cue_tag(self.scte35.cue)
+            self.scte35.cue_out = None
+
         else:
             self.scte35.cue_time = self.pid2pts(pid)
-            self._add_discontinuity()
             self.active_data.write("# Splice Immediate\n")
-        self.scte35.cue_tag = self.scte35.mk_cue_tag(self.scte35.cue)
-        self.scte35.cue_out = None
+            self._mk_cue_splice_point()
 
     def _mk_cue_splice_point(self):
         """
@@ -425,7 +449,7 @@ class X9K3(Stream):
             self.active_data.write(f"#EXTINF:{self.seg.seg_time},\n")
             self.active_data.write(seg_file + "\n")
             self.seg.seg_start = self.seg.seg_stop
-            self.seg.seg_stop += self.SECONDS
+            self.seg.seg_stop += self.seconds
             self.window.append(
                 (self.seg.seg_num, self.seg.seg_uri, self.active_data.getvalue())
             )
@@ -480,10 +504,11 @@ class X9K3(Stream):
         """
         rev = "\033[7m \033[1m"
         res = "\033[00m"
+        rev = res = ""
         now = time.time()
         gen_time = now - self.seg.init_time
         if not self.seg.seg_time:
-            self.seg.seg_time = self.SECONDS
+            self.seg.seg_time = self.seconds
         diff = self.seg.seg_time - gen_time
         self.seg.diff_total += diff
         furi = f"{rev}{self.seg.seg_uri}{res}"
@@ -496,52 +521,6 @@ class X9K3(Stream):
             if self.seg.diff_total > 0:
                 time.sleep(self.seg.seg_time)
 
-    def _is_key(self, pkt):
-        """
-        _is_key is key frame detection.
-        """
-
-        def _rai_flag(pkt):
-            """
-            random access indicator
-            """
-            return pkt[5] & 0x40
-
-        def _abc_flags(pkt):
-            """
-            0x80, 0x20, 0x8
-            """
-            return pkt[5] & 0xA8
-
-        def _nal(pkt):
-            """
-            \x65
-            """
-            return b"\x00\x00\x01\x65" in pkt
-
-        if _nal(pkt):
-            return True
-        if not self._afc_flag(pkt):
-            return False
-        if self._pcr_flag(pkt):
-            if _rai_flag(pkt):
-                return True
-        if _abc_flags(pkt):
-            return True
-        return False
-
-    @staticmethod
-    def _is_sps(pkt):
-        """
-        _is_sps parses h264 for profile and level
-        """
-        sps_start = b"\x00\x00\x01\x67"
-        if sps_start in pkt:
-            sps_idx = pkt.index(sps_start)
-            profile = pkt[sps_idx + 4]
-            level = pkt[sps_idx + 6]
-            print(f"Profile {profile} Level {level}")
-
     def _parse_pts(self, pkt, pid):
         """
         parse pts from pkt and store it
@@ -560,21 +539,23 @@ class X9K3(Stream):
             self._prgm_pts[prgm] = pts
             if not self.seg.seg_start:
                 self.seg.seg_start = self.as_90k(pts)
-                self.seg.seg_stop = self.seg.seg_start + self.SECONDS
+                self.seg.seg_stop = self.seg.seg_start + self.seconds
 
     def _parse(self, pkt):
         """
         _parse parses mpegts and
         writes the packet to self.active_segment.
         """
+
         pid = self._parse_info(pkt)
         self.chk_sidecar_cues(pid)
         if pid in self._pids["scte35"]:
             self.chk_stream_cues(pkt, pid)
-        # self._is_sps(pkt)
         if self._pusi_flag(pkt):
             self._parse_pts(pkt, pid)
-            if self._is_key(pkt):
+            if self.iframer.parse(pkt):
+                #      self.sps.is_sps(pkt)
+                self.load_sidecar(self.sidecar_file, pid)
                 self._mk_segment(pid)
                 if not self.start:
                     self.start = True
@@ -595,7 +576,7 @@ class X9K3(Stream):
         ]
         self._add_discontinuity()
         self._tsdata.seek(0)
-        self.loop()
+        return True
 
     def run(self):
         """
@@ -603,7 +584,8 @@ class X9K3(Stream):
         or else it calls decode()
         """
         if self.replay:
-            self.loop()
+            while True:
+                self.loop()
         else:
             self.decode()
 
@@ -614,27 +596,14 @@ def cli():
     for running X9K3  with command line args
     usage: x9k3 [-h] [-i INPUT] [-o OUTPUT_DIR] [-s SIDECAR] [-l] [-d] [-r] [-v]
 
-    optional arguments:
-      -h, --help            show this help message and exit
-      -i INPUT, --input INPUT
-                            Input source, like "/home/a/vid.ts" or "udp://@235.35.3.5:3535" or "https://futzu.com/xaa.ts"
-      -o OUTPUT_DIR, --output_dir OUTPUT_DIR
-                            Directory for segments and index.m3u8 ( created if it does not exist )
-      -s SIDECAR, --sidecar SIDECAR
-                            sidecar file of scte35 cues. each line contains PTS, Cue
-      -l, --live            Flag for a live event ( enables sliding window m3u8 )
-      -d, --delete          delete segments ( enables --live )
-      -r, --replay          Flag for replay (looping) ( enables --live and --delete )
-      -v, --version         Show version
-
     Two lines of code gives you a full X9K3 command line tool.
 
      from X9K3 import cli
      cli()
 
     """
-    x9 = X9K3()
-    x9.run()
+    stuff = X9K3()
+    stuff.run()
 
 
 if __name__ == "__main__":
